@@ -26,6 +26,7 @@
 // local includes
 #include "core/area.h"
 #include "core/page.h"
+#include "core/page_p.h"
 #include "core/annotations.h"
 #include "core/utils.h"
 #include "guiutils.h"
@@ -33,6 +34,7 @@
 #include "core/observer.h"
 #include "core/tile.h"
 #include "settings_core.h"
+#include "core/document_p.h"
 
 K_GLOBAL_STATIC_WITH_ARGS( QPixmap, busyPixmap, ( KIconLoader::global()->loadIcon("okular", KIconLoader::NoGroup, 32, KIconLoader::DefaultState, QStringList(), 0, true) ) )
 
@@ -51,14 +53,14 @@ inline QPen buildPen( const Okular::Annotation *ann, double width, const QColor
 }
 
 void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page * page,
-    int pixID, int flags, int scaledWidth, int scaledHeight, const QRect &limits )
+    Okular::DocumentObserver *observer, int flags, int scaledWidth, int scaledHeight, const QRect &limits )
 {
-        paintCroppedPageOnPainter( destPainter, page, pixID, flags, scaledWidth, scaledHeight, limits,
+        paintCroppedPageOnPainter( destPainter, page, observer, flags, scaledWidth, scaledHeight, limits,
                                    Okular::NormalizedRect( 0, 0, 1, 1 ), 0 );
 }
 
 void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okular::Page * page,
-    int pixID, int flags, int scaledWidth, int scaledHeight, const QRect &limits,
+    Okular::DocumentObserver *observer, int flags, int scaledWidth, int scaledHeight, const QRect &limits,
     const Okular::NormalizedRect &crop, Okular::NormalizedPoint *viewPortPoint )
 {
 	/* Calculate the cropped geometry of the page */
@@ -87,13 +89,13 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
     }
     destPainter->fillRect( limits, backgroundColor );
 
-    const bool hasTilesManager = ( pixID == PAGEVIEW_ID && page->hasTilesManager() );
+    const bool hasTilesManager = ( page->d->m_doc->m_tiledObserver == observer && page->hasTilesManager() );
     const QPixmap *pixmap = 0;
 
     if ( !hasTilesManager )
     {
         /** 1 - RETRIEVE THE 'PAGE+ID' PIXMAP OR A SIMILAR 'PAGE' ONE **/
-        pixmap = page->_o_nearestPixmap( pixID, scaledWidth, scaledHeight );
+        pixmap = page->_o_nearestPixmap( observer, scaledWidth, scaledHeight );
 
         /** 1B - IF NO PIXMAP, DRAW EMPTY PAGE **/
         double pixmapRescaleRatio = pixmap ? scaledWidth / (double)pixmap->width() : -1;
@@ -473,11 +475,15 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
                     }
 
                     const QPen linePen = buildPen( a, a->style().width(), a->style().color() );
+                    QBrush fillBrush;
+
+                    if ( la->lineClosed() && la->lineInnerColor().isValid() )
+                        fillBrush = QBrush( la->lineInnerColor() );
 
                     // draw the line as normalized path into image
                     drawShapeOnImage( backImage, path, la->lineClosed(),
                                       linePen,
-                                      QBrush(), pageScale ,Multiply);
+                                      fillBrush, pageScale ,Multiply);
 
                     if ( path.count() == 2 && fabs( la->lineLeadingForwardPoint() ) > 0.1 )
                     {
@@ -691,10 +697,14 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
                     painter.setPen( Qt::black );
                     painter.setFont( text->textFont() );
                     Qt::AlignmentFlag halign = ( text->inplaceAlignment() == 1 ? Qt::AlignHCenter : ( text->inplaceAlignment() == 2 ? Qt::AlignRight : Qt::AlignLeft ) );
-                    painter.scale( 1.0 * scaledWidth / page->width(), 1.0 * scaledHeight / page->height() );
-                    painter.drawText( 2, 2, image.width() - 2, image.height() - 2,
+                    const double invXScale = (double)page->width() / scaledWidth;
+                    const double invYScale = (double)page->height() / scaledHeight;
+                    painter.scale( 1 / invXScale, 1 / invYScale );
+                    painter.drawText( 2 * invXScale, 2 * invYScale,
+                                      (image.width() - 2) * invXScale,
+                                      (image.height() - 2) * invYScale,
                                       Qt::AlignTop | halign | Qt::TextWordWrap,
-                                      text->inplaceText() );
+                                      text->contents() );
                     painter.resetTransform();
                     painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
                     painter.end();
@@ -720,7 +730,7 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
                     // use it to colorize the icon, otherwise the icon will be
                     // "gray"
                     if ( a->style().color().isValid() )
-                        colorizeImage( scaledImage, a->style().color(), opacity );
+                        GuiUtils::colorizeImage( scaledImage, a->style().color(), opacity );
                     pixmap = QPixmap::fromImage( scaledImage );
 
                 // draw the mangled image to painter
@@ -900,7 +910,7 @@ void PagePainter::scalePixmapOnImage ( QImage & dest, const QPixmap * src,
 
 /** Private Helpers :: Image Drawing **/
 // from Arthur - qt4
-inline int qt_div_255(int x) { return (x + (x>>8) + 0x80) >> 8; }
+static inline int qt_div_255(int x) { return (x + (x>>8) + 0x80) >> 8; }
 
 void PagePainter::changeImageAlpha( QImage & image, unsigned int destAlpha )
 {
@@ -926,39 +936,6 @@ void PagePainter::changeImageAlpha( QImage & image, unsigned int destAlpha )
     }
 }
 
-void PagePainter::colorizeImage( QImage & grayImage, const QColor & color,
-    unsigned int destAlpha )
-{
-    // iterate over all pixels changing the alpha component value
-    unsigned int * data = (unsigned int *)grayImage.bits();
-    unsigned int pixels = grayImage.width() * grayImage.height();
-    int red = color.red(),
-        green = color.green(),
-        blue = color.blue();
-
-    int source, sourceSat, sourceAlpha;
-    for( register unsigned int i = 0; i < pixels; ++i )
-    {   // optimize this loop keeping byte order into account
-        source = data[i];
-        sourceSat = qRed( source );
-        int newR = qt_div_255( sourceSat * red ),
-            newG = qt_div_255( sourceSat * green ),
-            newB = qt_div_255( sourceSat * blue );
-        if ( (sourceAlpha = qAlpha( source )) == 255 )
-        {
-            // use destAlpha
-            data[i] = qRgba( newR, newG, newB, destAlpha );
-        }
-        else
-        {
-            // use destAlpha * sourceAlpha product
-            if ( destAlpha < 255 )
-                sourceAlpha = qt_div_255( destAlpha * sourceAlpha );
-            data[i] = qRgba( newR, newG, newB, sourceAlpha );
-        }
-    }
-}
-
 void PagePainter::drawShapeOnImage(
     QImage & image,
     const NormalizedPath & normPath,
